package shanghu

import (
	orm "duoduo/database"
	"github.com/shopspring/decimal"
	"time"
)

type ClientPayTrans struct {
	ID             int64           `gorm:"column:id;type:bigint(20);primary_key" json:"id"`                   // 主键
	RequestID      string          `gorm:"column:request_id;type:varchar(255)" json:"request_id"`             // 请求id，幂等性
	OutTradeNo     string          `gorm:"column:out_trade_no;type:varchar(255)" json:"out_trade_no"`         // 交易id
	MerchantCardID int64           `gorm:"column:merchant_card_id;type:bigint(20)" json:"merchant_card_id"`   // 商户卡id
	ClientOpenID   string          `gorm:"column:client_open_id;type:varchar(255)" json:"client_open_id"`     // 客户端openid
	Status         int             `gorm:"column:status;type:int(11)" json:"status"`                          // 1-未支付 2-支付成功 3-取消支付 4-退款
	Amount         decimal.Decimal `gorm:"column:amount;type:decimal(10,2)" json:"amount"`                    // 交易金额
	ThirdTradeNo   string          `gorm:"column:third_trade_no;type:varchar(255)" json:"third_trade_no"`     // 微信交易id
	PayTime        time.Time       `gorm:"column:pay_time;type:datetime(3);default:null" json:"pay_time"`     // 支付时间
	CreateBy       int64           `gorm:"column:create_by;type:bigint(20)" json:"create_by"`                 // 创建者
	UpdateBy       int64           `gorm:"column:update_by;type:bigint(20)" json:"update_by"`                 // 更新者
	CreatedAt      time.Time       `gorm:"column:created_at;type:datetime(3)" json:"created_at"`              // 创建时间
	UpdatedAt      time.Time       `gorm:"column:updated_at;type:datetime(3)" json:"updated_at"`              // 最后更新时间
	DeletedAt      time.Time       `gorm:"column:deleted_at;type:datetime(3);default:null" json:"deleted_at"` // 删除时间
}

func (m *ClientPayTrans) TableName() string {
	return "client_pay_trans"
}

func (m *ClientPayTrans) GetRequestNum() int {
	var count int

	tableCount := orm.ShMysql.Table(m.TableName()).Where("request_id = ? ", m.RequestID)
	tableCount.Count(&count)
	return count

}

func (m *ClientPayTrans) GetPayTransByTradeNo() (ClientPayTrans, error) {
	var doc ClientPayTrans

	table := orm.ShMysql.Table(m.TableName())
	table = table.Where("out_trade_no = ?  ", m.OutTradeNo)

	if err := table.Select("*").First(&doc).Error; err != nil {
		return doc, err
	}

	return doc, nil
}

func (u *ClientPayTrans) Create() (ClientPayTrans, error) {
	var doc ClientPayTrans
	var err error

	doc = *u

	err = orm.ShMysql.Table(u.TableName()).Create(&doc).Error
	if err != nil {
		return doc, err
	}

	return doc, nil
}

//var data biz.EbankAccountVa
//err := c.data.mysql.WithContext(ctx).Table(req.TableName()).Where("open_id = ? and pay_agency = ? and  account_no = ?", req.OpenID, req.PayAgency, req.AccountNo).Select("id").First(&data).Error
//
//if err != nil && !errors.Is(err, gorm.ErrRecordNotFound) {
//return err
//}
//req.ID = data.ID
//err = c.data.mysql.Clauses(clause.OnConflict{
//UpdateAll: true,
//}).Table(req.TableName()).Create(&req).Error
//if err != nil {
//return err
//}
//return nil

func (m *ClientPayTrans) UpdatePayTransByTradeNo() error {

	if err := orm.ShMysql.Table(m.TableName()).Model(&m).Where("out_trade_no = ? ", m.OutTradeNo).Updates(
		map[string]interface{}{
			"third_trade_no": m.ThirdTradeNo,
			"pay_time":       m.PayTime,
			"updated_at":     time.Now()}).Error; err != nil {
		return err
	}

	return nil
}
